/* ============================================================
 * File  : streamtuned.cpp
 * Author: Eric Giesselbach <ericgies@kabelfoon.nl>
 * Date  : 2004-02-12
 * Description : streamtuned gui
 *
 *
 * Copyright 2003 by Eric Giesselbach

 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

// development snapshot, many todo's

// mplayer stream recording
//mplayer rtsp://163.205.10.22:8080/broadcast/ksc1/redundant/nasatv.rm -dumpstream -dumpfile test
//tail -c 10000000000000 -f test |mplayer -

#include <iostream>

#include <qapplication.h>
#include <qsqldatabase.h>
#include <unistd.h>
#include <qpushbutton.h>
#include <qfont.h>
#include <qpainter.h>
#include <qlabel.h>
#include <qimage.h>
#include <qsettings.h>

#include "streamtuned.h"
#include "../libs/streambrowser.h"
#include "../libs/configelements.h"

using namespace std;

void alphaImage(QImage& dest, QImage& source, int fromX, int fromY)
{
  QRgb dest_rgb;
  QRgb source_rgb;
  int alpha,inv_alpha;

  int maxX = min( dest.width() , fromX + source.width() );
  int maxY = min( dest.height(), fromY + source.height() );

  for(int y=fromY; y<maxY; y++)
  {
    for(int x=fromX; x<maxX; x++)
    {
      source_rgb = source.pixel(x-fromX, y-fromY);
      dest_rgb   = dest.pixel(x,y);
      inv_alpha  = 255 - qAlpha(source_rgb);
      alpha      = qAlpha(source_rgb);


      dest.setPixel(x, y, qRgb( qRed(dest_rgb)*inv_alpha/255   + qRed(source_rgb)*alpha/255,
                                qGreen(dest_rgb)*inv_alpha/255 + qGreen(source_rgb)*alpha/255,
                                qBlue(dest_rgb)*inv_alpha/255  + qBlue(source_rgb)*alpha/255
                              )
                  );
    }
  }

  dest.setAlphaBuffer(false);
}

ConfigWriter::ConfigWriter()
{    
    setPath( ""TARGET, ""TARGET, QSettings::User );
    insertSearchPath( QSettings::Unix, QString(getenv("HOME")) + "/."SUBPATH );
}

uint ConfigWriter::readUInt(QString key, uint defVal)
{
    QString strTemp = "";
    uint uintTemp = 0;
    QString str = "%1";

    bool *success = NULL;
        
    uintTemp = defVal;
    
    strTemp = readEntry( key, "-1-1-1-1", success);
    if (strTemp != "-1-1-1-1") 
      uintTemp = strTemp.toUInt(success, 10);
    else
      writeEntry( key, str.arg(defVal, 0, 10) );
    
    return uintTemp;
}

QColor ConfigWriter::readColor(QString key, uint defColor)
{
    QString strTemp = "";
    uint uintTemp = 0;
    QString str = "%1";
    
    bool *success = new bool;
    
    strTemp = readEntry( key, "-1-1-1-1", success);
    if (strTemp != "-1-1-1-1")
      uintTemp = strTemp.toUInt(success, 16);
    else
      writeEntry( key, str.arg(defColor, 0, 16).rightJustify(6, '0') );
    
    if (*success) 
      return QColor(uintTemp); 
    else 
      return QColor(defColor);
      
}

// ----------------------------------------------------------------------


StreamTuned::StreamTuned( QWidget *parent, const char *name )
        : QWidget( parent, name, WStyle_Customize | WStyle_NoBorder | WStyle_Tool)
{
    setMinimumSize( 573, 149 );
    setMaximumSize( 573, 149 );

    storageBin = new ReposStorage();  // creates .streamtuned directory if needed

    harvesterBusy = false;
    switchLoopDetect = false; 
    // ...
            
    guiParameters = new GuiParameters();
    
    guiParameters->textLoColor        = settings.readColor( "/settings/colors/normFont"         , 0xcfcf01);
    guiParameters->textSelectedColor  = settings.readColor( "/settings/colors/selectedFont"     , 0xffff33);
    guiParameters->textHighlightColor = settings.readColor( "/settings/colors/hightlightFont"   , 0xffffff);
    guiParameters->gaugeColor         = settings.readColor( "/settings/colors/gaugeFont"        , 0x0000ff);
    guiParameters->playerBoxEdge      = settings.readColor( "/settings/colors/playerBoxEdge"    , 0x0000ff);
    guiParameters->playerBoxFont      = settings.readColor( "/settings/colors/playerBoxFont"    , 0x0000ff);
    guiParameters->harvesterBoxEdge   = settings.readColor( "/settings/colors/harvesterBoxEdge" , 0x0000ff);
    guiParameters->harvesterBoxFont   = settings.readColor( "/settings/colors/harvesterBoxFont" , 0x0000ff);
    guiParameters->sampleTextWidth    = settings.readUInt( "/settings/fonts/sampleTextWidth"    , 125     );

    selectFont();
    
    setIcon( QString(PREFIX"/share/streamtuned/s_icon.png") );

    loadActualBackground();
    
    setFocusPolicy( StrongFocus );

    streamPict.load   ( QString(PREFIX"/share/streamtuned/i_stream.png") );
    urlPict.load      ( QString(PREFIX"/share/streamtuned/i_url.png") );
    infoPict.load     ( QString(PREFIX"/share/streamtuned/i_info.png") );
    htmlPict.load     ( QString(PREFIX"/share/streamtuned/i_html.png") );
    unknownPict.load  ( QString(PREFIX"/share/streamtuned/i_unknown.png") );
    scheduledPict.load( QString(PREFIX"/share/streamtuned/i_scheduled.png") );
    recordingPict.load( QString(PREFIX"/share/streamtuned/i_recording.png") );
    recordedPict.load ( QString(PREFIX"/share/streamtuned/i_recorded.png") );
    downloadPict.load ( QString(PREFIX"/share/streamtuned/i_download.png") );
    generalPict.load  ( QString(PREFIX"/share/streamtuned/i_general.png") );
    headerPict.load   ( QString(PREFIX"/share/streamtuned/i_header.png") );
    markedPict.load   ( QString(PREFIX"/share/streamtuned/i_marked.png") );  // myth
    
    mousePosition = QPoint(-1000, -1000);
    streamTunedConfig = 0;

    // gui elements common to all displays
    logo = new Logo( this, 10, 2, 120, 16 );
    folderLabels.insert(0, new FolderLabel( this, guiParameters,  95, 115, 95, 12  ) );
    folderLabels.insert(1, new FolderLabel( this, guiParameters, 195, 115, 95, 12 ) );
    folderLabels.insert(2, new FolderLabel( this, guiParameters, 295, 115, 95, 12 ) );

    browserDisplay = new BrowserDisplay( this, guiParameters, QRect( 62, 21, 444, 90 ), folderLabels );
    streamDisplay  = new StreamDisplay ( this, guiParameters, QRect( 62, 21, 444, 90 ), folderLabels );      

    // for fft
    sampler = 0;
    
    // Misc
    userMessageBox = new MessageLabel( this, guiParameters, 70, 60, 436, 30 );

    streamLed = new Led(this, 520, 19, true);
    streamLed->setColor( green );
    streamLed->setLightness(50);

    recLed = new RecLed(this, 46, 82);

    harvesterLed = new Led(this, 512, 92, false);
    harvesterLed->setLightness(50);

    // mouse sensitive area's
    itemButtonArea    = QRect(511, 17, 50, 92);
    folderButtonArea  = QRect(459, 111, 116, 19);
    globalButtonArea  = QRect(10, 17, 52, 100);
    controlButtonArea = QRect(535, 2, 30, 14);

    videoRect         = QRect(100, 100, 400, 300); // todo
    //videoMaxRect      = QRect(0, 0, 800, 600); // todo
    videoMaxRect      = QApplication::desktop()->rect();

    viewerRect         = QRect(100, 100, 400, 300); // todo
    viewerMaxRect      = QApplication::desktop()->rect();
    
    streamStorage = new StreamStorage();
    
    connect (
               streamStorage,
               SIGNAL( storageEvent(int, int, bool ) ),
               this,
               SLOT  ( slotStorageEvent(int, int, bool ) )
             );

    streamBrowser = new StreamBrowser(0, this, streamStorage);
    streamBrowser->setVideoRect(videoRect);
    streamBrowser->setVideoMaxRect(videoMaxRect);
    streamBrowser->setViewerRect(viewerRect);
    streamBrowser->setViewerMaxRect(viewerMaxRect);
    streamBrowser->setItemDisplayFolded( -1, false );
    streamBrowser->setFolderDisplayFolded( -1, false );

    browserHandler  = new BrowserHandler( browserDisplay, streamBrowser ); 
    //viewerHandler     = new ViewerHandler   ( browserDisplay, streamBrowser );
    streamHandler   = new StreamHandler ( streamDisplay , streamBrowser ); 
    currentHandler  = 0;
    previousHandler = 0;
    switchHandler(browserHandler);
    
    connect (
              streamBrowser,
              SIGNAL(eventItemTreeSwitchedTo(int)),
              this,
              SLOT(slotItemTreeSwitchedTo(int))
            );

    connect (
              streamBrowser,
              SIGNAL(eventHarvesterBusy(bool, QString)),
              this,
              SLOT(slotHarvesterBusy(bool, QString))
            );

    connect (
              streamBrowser,
              SIGNAL(eventVideoActive(bool)),
              this,
              SLOT(slotVideoActive(bool))
            );

    connect (
              streamBrowser,
              SIGNAL(eventUserMessage()),
              this,
              SLOT(slotUserMessage())
            );

    connect (
              streamBrowser,
              SIGNAL( eventValuesUpdated(int) ),
              this,
              SLOT( slotValuesUpdated(int) )
            );

    connect (
              streamBrowser,
              SIGNAL( eventRecorderActive(bool) ),
              this,
              SLOT( slotRecorderActive(bool) )
            );

    connect (
              streamBrowser,
              SIGNAL( eventFFTReady(Spectrum*) ),
              streamDisplay->fft,
              SLOT( setPercentage(Spectrum*) )
            );
            
    connect (
              browserDisplay,
              SIGNAL( getIconPixmap( QPixmap&, QChar) ),
              this,
              SLOT( slotGetIconPixmap( QPixmap&, QChar) )
            );
            
    connect (
              streamHandler,
              SIGNAL( switchHandler(int) ),
              this,
              SLOT( slotSwitchHandler(int) )
            );
            
    connect (
              browserHandler,
              SIGNAL( switchHandler(int) ),
              this,
              SLOT( slotSwitchHandler(int) )
            );
                

    streamBrowser->enableFFT(20);

    setDisplayStatus("browse");
    videoActive = false;
    pollResetDisplayCount = 0;

    itemTreeLoaded = StreamBrowser::streams; // myth
    
    ValueList values(9);
    QString error;
    
    bool loaded = storageBin->getDefaultRepository(values);

    if (loaded)
      loaded = streamStorage->selectStorage(100, values);

    if (!loaded)
    {
       cerr << "cannot open default repository" << endl;
       qApp->exit(-1);
       return;
    }
    
    storageBin->checkParsers(); // (stream list parsers) oops, method should not be in storageBin...
        
    streamBrowser->initReady();
    
}

StreamTuned::~StreamTuned()
{
  if (sampler)
    delete(sampler);
  delete ( streamBrowser );
  if (streamStorage)
    delete ( streamStorage );
  if (storageBin)
    delete ( storageBin );
  if (guiParameters)
    delete ( guiParameters );
  if (browserHandler)
    delete ( browserHandler );
  if (streamHandler)
    delete ( streamHandler );
  //db->close();
}

void StreamTuned::loadActualBackground()
{
       QImage image1;
        image1.load( QString(PREFIX"/share/streamtuned/s_frame.png") );
       QImage image2;
        image2.load( QString(PREFIX"/share/streamtuned/s_picture.png") );
       QImage image3;
        image3.load( QString(PREFIX"/share/streamtuned/s_frame.png") );
       QImage image4;
        image4.load( QString(PREFIX"/share/streamtuned/s_glass.png") );

       // can be removed after qt v3.xxx?
       alphaImage(image1, image2, 67, 19);  // position picture in image with size of frame
       alphaImage(image1, image3, 0, 0);    // mask correctly positioned picture with radio frame
       alphaImage(image1, image4, 67, 19);  // blend display

       pict.convertFromImage(image1);

    updateBackground();

}

void StreamTuned::selectFont()
{
    QFont defaultFont = font();
    //defaultFont.setWeight(QFont::DemiBold);

    int pointSize = defaultFont.pointSize();
    int pointSizeHi = 0;

    if ( pointSize == -1 )
    {
      cerr << "font pointSize -1" << endl;
      return;
    }

    QFontMetrics metric( defaultFont );
    int renderWidthHi = metric.width( "64 Kbps 44 KHz stereo" ); //120

    // increase fontsize until render width > 120 (if not already)
    while ( renderWidthHi < int(guiParameters->sampleTextWidth) )
    {
      defaultFont.setPointSize( ++pointSize );
      metric = QFontMetrics( defaultFont );
      renderWidthHi = metric.width( "64 Kbps 44 KHz stereo" );
    }

    // determine fontsize direct above and below 120 reference
    int renderWidthLo = renderWidthHi;
    while ( renderWidthLo > int(guiParameters->sampleTextWidth) )
    {
      pointSizeHi = pointSize;
      renderWidthHi = renderWidthLo;
      defaultFont.setPointSize( --pointSize );
      metric = QFontMetrics( defaultFont );
      renderWidthLo = metric.width( "64 Kbps 44 KHz stereo" );
    }

    if ( (int(guiParameters->sampleTextWidth) - renderWidthLo) > (renderWidthHi - int(guiParameters->sampleTextWidth)) )
      defaultFont.setPointSize(pointSizeHi);

    setFont( defaultFont );
}


void StreamTuned::slotStorageEvent(int ident, int eventType, bool error)
{
   if ( ident != 100 ) return;

   if (error)
   {
     reportEvent( streamStorage->getLastError() );
     return;
   }

   QString err;

   switch ( eventType )
   {
     case StreamStorage::selected:
        //cout << "selected" << endl;
        //if ( streamStorage->getAccessType() == StreamStorage::web )
          if ( !streamStorage->loadList(100, err) )
            reportEvent ( err );
     break;

     case StreamStorage::loaded:
     break;

     default:
     break;
   }
}

bool StreamTuned::copyFile(const QString& source, const QString& dest)
{
  QFile src(source);
  QFile dst(dest);

  if ( !src.open(IO_ReadOnly) )
    return false;
  if ( !dst.open(IO_ReadWrite) )
    return false;

  QTextStream srcStream( &src );
  QTextStream dstStream( &dst );
  QString line;

  while ( !srcStream.eof() )
  {
    line = srcStream.readLine();
    dstStream << line << endl;
  }

  dst.close();
  src.close();

  return true;
}

bool StreamTuned::createStorageRepos(const QString& dest, const QString& path)
{
  QFile dst(dest);

  if ( !dst.open(IO_ReadWrite) )
    return false;

  QTextStream dstStream( &dst );

  dstStream << endl;
  dstStream << "[item]" << endl;
  dstStream << "*" << endl;
  dstStream << "file" << endl;
  dstStream << "default" << endl;
  dstStream << path << "/.streamtuned/streams.res" << endl;
  dstStream << endl;

  dst.close();

  return true;
}


void StreamTuned::updateBackground()
{
    //QPixmap pix(size());
    //pix.load( QString(PREFIX"/share/streamtuned/radiobg.png") );
    //setPaletteBackgroundPixmap(pix);
    setPaletteBackgroundPixmap(pict);
}

void StreamTuned::focusInEvent ( QFocusEvent * )
{
  logo->parentHasFocus(true);
}

void StreamTuned::focusOutEvent ( QFocusEvent * )
{
  logo->parentHasFocus(false);
}

void StreamTuned::keyPressEvent( QKeyEvent *k )
{
    Command key_command = nada;
    
    switch ( k->key() )
    {
        case Key_Up:
            key_command = previtem;
            break;
        case Key_Down:
            key_command = nextitem;
            break;
        case Key_Left:
            key_command = prevfolder;
            break;
        case Key_Right:
            key_command = nextfolder;
            break;
        case Key_Return:
            key_command = selectitem;
            break;
        case Key_Escape:
            key_command = escape;
            break;
        case Key_End:
            key_command = end;
            break;
        case Key_D:
            key_command = dump;
            break;
        case Key_F:
            key_command = fullscreen;
            break;
        case Key_P:
            key_command = pauseitem;
            break;
        case Key_R:
            key_command = record;
            break;
        case Key_S:
            key_command = stoprecord;
            break;
        case Key_Q:
            key_command = stopallrecord;
            break;
        case Key_BracketLeft:
            key_command = voldn;
            break;
        case Key_BracketRight:
            key_command = volup;
            break;
        case Key_M:
            key_command = mark;
            break;
        case Key_Y:
            key_command = storemarked;
            break;
        case Key_I:
            key_command = inspect;
            break;
        case Key_Less:
            key_command = rewindstream;
            break;
        case Key_Greater:
            key_command = forwardstream;
            break;
    }
    
    // process key input
    //if ( key_command == inspect && currentHandler != viewerHandler)
    //{
    //   switchHandler(viewerHandler);
    //}
    //  else
         if (! currentHandler->handlePressedKey(key_command) &&
             ! streamBrowser->handlePressedKey(key_command)      )
           k->ignore();
}

void StreamTuned::slotSwitchHandler(int key)  // switch request from handler, decision based on key
{
   //cout << "slotswitch" << endl;
   
   if (switchLoopDetect)
     cerr << TARGET" error: handler switch loop detected" << endl;
   
   if (previousHandler && !switchLoopDetect) 
   {
     switchLoopDetect = true;
     switchHandler(previousHandler);
     currentHandler->handlePressedKey(key);
   }
   
   switchLoopDetect = false;
}

void StreamTuned::switchHandler(GenericHandler *handler)  // switch
{
   //cout << "switch to: " << handler->className() << endl;
   if ( currentHandler && handler != currentHandler ) currentHandler->deactivate();
   handler->activate();
   previousHandler = currentHandler;
   currentHandler  = handler;
}

int getItemButtonIndex(int /*x*/, int y)
{
   if ( y < 37)  return 0;
   if ( y < 56 ) return 1;
   return 2;
}

int getFolderButtonIndex(int x, int /*y*/)
{
   if ( x < 57)  return 0;
   return 1;
}

int getGlobalButtonIndex(int x, int y)
{
   if ( y < 20  && x < 28 )  return 0;  // play
   if ( y < 20 )  return 10;            // inspect
   
   if ( y < 40  && x < 28 )  return 1;
   if ( y < 40  )            return 2;
   if ( y < 60  && x < 28 )  return 3;
   if ( y < 60  )            return 4;
   if ( y < 80  && x < 28 )  return 5;
   if ( y < 80  )            return 6;
   if ( y < 100 && x < 28 )  return 7;
   return 8;
}

int getControlButtonIndex(int x, int /*y*/)
{
   if ( x < 18 )  return 0;
   return 1;
}

void StreamTuned::mouseReleaseEvent(  QMouseEvent * e )
{
   if (e)
     mousePosition = QPoint(-1000, -1000);
}

void StreamTuned::mouseMoveEvent(  QMouseEvent * e )
{
   if ( mousePosition != QPoint(-1000, -1000) )
   {
     move( pos() + ( e->pos() - mousePosition ) );
   }
}

void StreamTuned::mousePressEvent( QMouseEvent * e )
{
    int x, y, index;

    mousePosition = e->pos();

    if ( controlButtonArea.contains( e->pos() ) )
    {
        x = e->x() - controlButtonArea.left();
        y = e->y() - controlButtonArea.top();
        index = getControlButtonIndex(x, y);

        switch ( index )
        {
          case 0:
              showMinimized();
              break;
          case 1:
              close();
              break;
        }
    }
    else if ( itemButtonArea.contains( e->pos() ) )
    {
        x = e->x() - itemButtonArea.left();
        y = e->y() - itemButtonArea.top();
        index = getItemButtonIndex(x, y);

        switch ( index )
        {
          case 0:
              currentHandler->handlePressedKey( previtem );
              break;
          case 1:
              currentHandler->handlePressedKey( selectitem );
              break;
          case 2:
              currentHandler->handlePressedKey( nextitem );
              break;
        }
    }
      else if ( folderButtonArea.contains( e->pos() ) )
    {
        x = e->x() - folderButtonArea.left();
        y = e->y() - folderButtonArea.top();
        index = getFolderButtonIndex(x, y);

        switch ( index )
        {
          case 0:
              currentHandler->handlePressedKey( prevfolder );
              break;
          case 1:
              currentHandler->handlePressedKey( nextfolder );
              break;
        }
    }
      else if ( globalButtonArea.contains( e->pos() ) )
    {
        x = e->x() - globalButtonArea.left();
        y = e->y() - globalButtonArea.top();
        index = getGlobalButtonIndex(x, y);

        switch ( index )
        {
          case 0:
              currentHandler->handlePressedKey( selectitem );
              break;
          case 1:
              currentHandler->handlePressedKey( end );
              break;
          case 2:
              currentHandler->handlePressedKey( pauseitem );
              break;
          case 3:
              currentHandler->handlePressedKey( voldn );
              break;
          case 4:
              currentHandler->handlePressedKey( volup );
              break;
          case 5:
              currentHandler->handlePressedKey( stoprecord );
              break;
          case 6:
              currentHandler->handlePressedKey( record );
          case 7:
              if (!streamTunedConfig)
              {
                streamTunedConfig =
                  new StreamTunedConfig( this, "StreamTuned Configuration", streamStorage, storageBin);
                streamTunedConfig->show();
              }
              else
              {
                streamTunedConfig->show();
                streamTunedConfig->showNormal();
                streamTunedConfig->raise();
                streamTunedConfig->setActiveWindow();
              }
              break;
          case 8:
              currentHandler->handlePressedKey( fullscreen );
              break;
          case 10:
              currentHandler->handlePressedKey( inspect );
              break;
        }
    }
      //else
       //item2->setText("");
}


void StreamTuned::slotItemTreeSwitchedTo(int treeLoaded)  // myth
{
      itemTreeLoaded = (StreamBrowser::ItemTreeLoaded)treeLoaded; 
      
      if ( itemTreeLoaded == StreamBrowser::streams ) 
        browserHandler->setTitleColor(white);
      if ( itemTreeLoaded == StreamBrowser::harvester ) 
        browserHandler->setTitleColor(green);
            
      if ( itemTreeLoaded == StreamBrowser::harvester )
        harvesterLed->setLightness(200);
      else
        harvesterLed->setLightness(50);
}

void StreamTuned::slotHarvesterBusy(bool busy, QString message)
{
   //cout << "st:harvesterbusy" << busy << "message" << endl;

   streamHandler->harvesterBusySignal(busy, message);
   
   harvesterBusy = busy; // if true keep display status "audio"

  if ( busy )
  {
      harvesterLed->setBlinking(true);
      //if ( pollResetDisplay(false) )
         setDisplayStatus("audio");
      //harvesterLed->setLightness(200);
  }
  else
  {
     if ( itemTreeLoaded == StreamBrowser::harvester )
       harvesterLed->setLightness(200);
     else
       harvesterLed->setLightness(50);
  }
}

void StreamTuned::slotVideoActive(bool active)
{
   // remove from compile warnings
   videoActive = active;
}

void StreamTuned::slotUserMessage()
{
   userMessageBox->setText( streamBrowser->getUserMessage() );
   userMessageBox->show();
}

void StreamTuned::reportEvent(QString msg)
{
   userMessageBox->setText( msg );
   userMessageBox->show();
}

bool StreamTuned::pollResetDisplay(bool initialize)
{
   if ( initialize )
     pollResetDisplayCount = 5;

   if ( --pollResetDisplayCount < 0 )
   {
     pollResetDisplayCount = 0;
     return true;
   }
     else return false;
}

void StreamTuned::setDisplayStatus(const QString& status)
{
   // this sets a delay for routines that want to call
   // setDisplayStatus with a delay
   pollResetDisplay(true);

   // things to clear on any display event
   userMessageBox->hide();

   if ( status == displayStatus ) 
     return;
     
   // things to clear on actual display status change

   
   if ( status == "audio" || status == "video" )
   {
      switchHandler(streamHandler);
   }
   else if ( status == "browse" )
   {
      switchHandler(browserHandler);
   }

   displayStatus = status;
}



void StreamTuned::slotGetIconPixmap(  QPixmap& pix, QChar prefix )
{
   //cout << "prefix: " << prefix << endl;
   
   switch (prefix)
   {
     case '~':
       pix = streamPict;
     break;
     case '-':
       pix = unknownPict;
     break;
     case ' ':
       pix = headerPict;
     break;
     case '_':
       pix = urlPict;
     break;
     case 'I':
       pix = infoPict;
     break;
     case 'H':
       pix = htmlPict;
     break;
     case 'R':
       pix = recordingPict;
     break;
     case 'S':
       pix = scheduledPict;
     break;
     case '#':
       pix = recordedPict;
     break;
     case 'D':
       pix = downloadPict;
     break;
     case '>': // header (keeptopline=1)
       pix = headerPict;
     break;
     case 'E': // empty line
       pix = headerPict;
     break;
     case '.': // default item icon
       pix = generalPict;
     break;
     case '!': // marked item
       pix = markedPict;  // myth
     break;
     default:
       pix = generalPict;
     break;
   }   
}



void StreamTuned::slotValuesUpdated(int updatedType)
{
   //cout << "st:valuesupdated" << updatedType << endl;
   QString value;
   
   switch( updatedType )
   {
     case StreamBrowser::streamstatus:
       streamHandler->updateStreamStatusValues();
    
          value = streamBrowser->getStreamParameter(StreamBrowser::streamStatusStr);
      
          if ( value != "idle" || harvesterBusy )
            setDisplayStatus("audio");
          else
            setDisplayStatus("browse");
      
          if ( currentStreamStatus != value )
          {
            if ( value == "buffering" )
              streamLed->setBlinking(true);
            else
            if ( value == "playing" )
              streamLed->setLightness(245);
            else
              streamLed->setLightness(50);
      
            currentStreamStatus = value;
          }
       

       break;
     case StreamBrowser::streamplay:
       streamHandler->updateStreamPlayValues();
       if ( pollResetDisplay(false) )
         setDisplayStatus("audio");
       break;
     case StreamBrowser::folderlist:
       browserHandler->getFolderList();
       browserHandler->getItemList();
       setDisplayStatus("browse");
       break;
     case StreamBrowser::itemlist:
       browserHandler->getItemList();
       setDisplayStatus("browse");
       break;
     default:
       break;
   }
}

void StreamTuned::slotRecorderActive(bool active)
{
   recLed->setIndicator(active);
}

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------


BrowserDisplay::BrowserDisplay(QWidget *parent, GuiParameters* params, QRect rect, QIntDict<FolderLabel> &labels) : QWidget(parent)
{
    guiParameters = params;
    this->folderLabels = labels;
    
    hide();
    
    setBackgroundOrigin(QWidget::WindowOrigin);
    setGeometry(rect);
    
    titleLabel = new ItemLabel( this, guiParameters, 10, 0, 420, 15 );
    
    itemLabels.insert(0, new ItemLabel( this, guiParameters, 25, 15, 420, 15 ) );
    itemLabels.insert(1, new ItemLabel( this, guiParameters, 25, 30, 420, 15 ) );
    itemLabels.insert(2, new ItemLabel( this, guiParameters, 25, 45, 420, 15 ) );
    itemLabels.insert(3, new ItemLabel( this, guiParameters, 25, 60, 420, 15 ) );
    itemLabels.insert(4, new ItemLabel( this, guiParameters, 25, 75, 420, 15 ) );

    itemIcons.insert(0, new ItemIcon( this, guiParameters, 10, 15, 15, 15 ) );
    itemIcons.insert(1, new ItemIcon( this, guiParameters, 10, 30, 15, 15 ) );
    itemIcons.insert(2, new ItemIcon( this, guiParameters, 10, 45, 15, 15 ) );
    itemIcons.insert(3, new ItemIcon( this, guiParameters, 10, 60, 15, 15 ) );
    itemIcons.insert(4, new ItemIcon( this, guiParameters, 10, 75, 15, 15 ) ); 
}

BrowserDisplay::~BrowserDisplay()
{
}

uint BrowserDisplay::getItemCount()
{
   return 5;
}

uint BrowserDisplay::getFolderCount()
{
   return 3;
}

void BrowserDisplay::setItemIcon(int index, QChar& idch)
{
   QPixmap pix;
   
   emit getIconPixmap( pix, idch );
   itemIcons[index]->setPixmap(pix);
}

void BrowserDisplay::setItemLabel(int index, QString& txt, bool selected)
{
   if ( txt ) 
     itemLabels[index]->setText(txt);
   itemLabels[index]->setSelected(selected);
}

void BrowserDisplay::setFolderLabel(int index, QString& txt, bool selected)
{
   if ( txt ) 
     folderLabels[index]->setText(txt);
   folderLabels[index]->setSelected(selected);
}

void BrowserDisplay::setTitleColor(QColor color, bool bold)
{
    titleLabel->setColor( color, bold );
}

void BrowserDisplay::setTitleLabel(QString& txt)
{
   titleLabel->setText(txt);
}





StreamDisplay::StreamDisplay(QWidget *parent, GuiParameters* params, QRect rect, QIntDict<FolderLabel> &labels) 
             : QWidget(parent)
{
    guiParameters = params;
    this->folderLabels = labels;
    
    hide();
    
    setBackgroundOrigin(QWidget::WindowOrigin);
    setGeometry(rect);
    
    titleLabel = new ItemLabel( this, guiParameters, 10, 0, 420, 15 );
    
    setGeometry( 62, 21, 444, 107 );
    setBackgroundOrigin(QWidget::WindowOrigin);

    // play info bottom
    streamStatusLabel = new ItemLabel( this, guiParameters, 40, 94, 40, 13 );
    streamStatusLabel->setText("status:");
    streamStatus      = new ItemLabel( this, guiParameters, 90, 94, 80, 13 );
    streamStatus->setHighLight( true );

    streamTimeLabel = new ItemLabel( this, guiParameters, 180, 94, 40, 13 );
    streamTimeLabel->setText("time:");
    streamTime      = new ItemLabel( this, guiParameters, 220, 94, 80, 13 );
    streamTime->setHighLight( true );

    // play info gauges
    streamCacheLabel = new ItemLabel( this, guiParameters, 355, 33, 80, 12);
    streamCacheLabel->setText("buffer");
    streamCacheGauge = new Gauge( this, guiParameters, 355, 45, 82, 10);

    streamStabilityLabel = new ItemLabel( this, guiParameters, 355, 58, 80, 12);
    streamStabilityLabel->setText("stability");
    streamStabilityGauge = new Gauge( this, guiParameters, 355, 70, 82, 10);

    fft = new FFTBox( this, guiParameters, 20, 355, 0, 80, 30);

    // stream properties
    streamName         = new ItemLabel( this,   guiParameters, 11,  2, 335, 15 );
    streamDescr        = new ItemLabel( this,   guiParameters, 11, 20, 335, 15 );
    streamAudioQuality = new ItemLabel( this,   guiParameters, 11, 38, 335, 15 );
    streamAudioFormat  = new ItemLabel( this,   guiParameters, 11, 56, 335, 15 );
    streamVideoFormat  = new ItemLabel( this,   guiParameters, 11, 74, 335, 15 );

    playerMessageBox    = new PlayerMessageBox   ( this, guiParameters, 60, 15, 340 );
    harvesterMessageBox = new HarvesterMessageBox( this, guiParameters, 99, 69, 340 );

    streamName->setSelected        ( true );
    streamDescr->setSelected       ( true );
    streamAudioQuality->setSelected( true );
    streamAudioFormat->setSelected ( true );
    streamVideoFormat->setSelected ( true );
}


uint StreamDisplay::getFolderCount()
{
   return 3;
}


void StreamDisplay::setFolderLabel(int index, QString& txt, bool selected)
{
   if ( txt ) 
     folderLabels[index]->setText(txt);
   folderLabels[index]->setSelected(selected);
}

void StreamDisplay::setTitleColor(QColor color, bool bold)
{
    titleLabel->setColor( color, bold );
}

void StreamDisplay::setTitleLabel(QString& txt)
{
   titleLabel->setText(txt);
}

    

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------

BrowserHandler::BrowserHandler(BrowserDisplay *browserDisplay, StreamBrowser *streamBrowser)
              : GenericHandler()
{
    this->browserDisplay = browserDisplay;
    this->streamBrowser  = streamBrowser;  
    setTitleColor(white);
    getFolderList();
    getItemList();
}
      
bool BrowserHandler::handlePressedKey( int key )
{
   return streamBrowser->handlePressedKey(key);
}

void BrowserHandler::activate()
{ 
   browserDisplay->show();
}

void BrowserHandler::deactivate()
{
   browserDisplay->hide();
}

void BrowserHandler::setTitleColor( QColor color ) 
{
   browserDisplay->setTitleColor( color, true );   
}

void BrowserHandler::getItemList()
{
    QStringList list;
    QString caption;
    int cnt;
    int cursor;
    QChar prefix;
    bool a,b;
    //cout << "getitemlist" << endl;
    
    caption = streamBrowser->getCurrentFolderCaption();
    
    browserDisplay->setTitleLabel( caption );  

    cursor = streamBrowser->getDisplayItemList(browserDisplay->getItemCount(), list, a, b);
    cnt = list.count();

    for (int i = 0; i < cnt; i++ )
    {
      caption = *list.at(i);
      prefix = caption.at(0);
      caption = caption.remove(0, 1);
      
      browserDisplay->setItemIcon( i, prefix );
      browserDisplay->setItemLabel( i, caption, false );
    }
    
    for (uint i = cnt; i < browserDisplay->getItemCount(); i++ )
    {
      prefix = 'E';
      caption = "";
      browserDisplay->setItemIcon( i, prefix );
      browserDisplay->setItemLabel( i, caption, false );
    }

    if ( cursor >= 0 )
    {
      caption = QString::null;  
      browserDisplay->setItemLabel( cursor, caption, true );
    }

}


void BrowserHandler::getFolderList()
{
    QStringList list;
    int cnt;
    int cursor;
    bool a,b;
    
    //cout << "getfolderlist" << endl;
    
    cursor = streamBrowser->getDisplayFolderList(browserDisplay->getFolderCount(), list, a, b);
    cnt = list.count();
    
    for (int i = 0; i < cnt; i++ )
    {
      browserDisplay->folderLabels[i]->setText( *list.at(i) );
      browserDisplay->folderLabels[i]->setSelected( false );
    }
    for (uint i = cnt; i < browserDisplay->getFolderCount(); i++ )
    {
      browserDisplay->folderLabels[i]->setText("");
      browserDisplay->folderLabels[i]->setSelected( false );
    }

    if ( cursor >= 0 )
      browserDisplay->folderLabels[cursor]->setSelected(true);
}



//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------

StreamHandler::StreamHandler(StreamDisplay *streamDisplay, StreamBrowser *streamBrowser)
              : GenericHandler()
{
    this->streamDisplay = streamDisplay;
    this->streamBrowser = streamBrowser;  
}
      
bool StreamHandler::handlePressedKey( int key )
{
   // if (key == previtem || key == nextitem || key==prevfolder || key == nextfolder) 
//      emit switchHandler(key);
     return streamBrowser->handlePressedKey(key);
    
    return true;
}

void StreamHandler::activate()
{
   streamDisplay->harvesterMessageBox->setMessage("");
   streamDisplay->playerMessageBox->clearMessages();
   streamDisplay->show();
}

void StreamHandler::deactivate()
{
   streamDisplay->hide();
}

void StreamHandler::setTitleColor(QColor color)
{
   streamDisplay->setTitleColor(color, true);
}

void StreamHandler::harvesterBusySignal(bool /*busy*/, QString& message)
{
   streamDisplay->harvesterMessageBox->setMessage( message );
}

void StreamHandler::updateStreamStatusValues()
{
    QString value;
    QString m_audioCh, m_audioSps, m_audioBps;
    QString m_videoQ;

    value = streamBrowser->getStreamParameter(StreamBrowser::streamName);
    streamDisplay->streamName->setText( value );
    value = streamBrowser->getStreamParameter(StreamBrowser::streamDescr);
    if (value == "")
      value = streamBrowser->getStreamParameter(StreamBrowser::streamUrl);
    streamDisplay->streamDescr->setText( value );
    value = streamBrowser->getStreamParameter(StreamBrowser::streamStatusStr);
    streamDisplay->streamStatus->setText( value );

    value = streamBrowser->getStreamParameter(StreamBrowser::streamAudioFormat);
    //check if ( videoActive )
      value = "Audio codec: " + value;
    streamDisplay->streamAudioFormat->setText( value );


    value = streamBrowser->getStreamParameter(StreamBrowser::streamAudioChannels);
    if ( value == "1" ) m_audioCh = "mono";
    else if ( value == "2" ) m_audioCh = "stereo";
    else m_audioCh = value + " ch";
    value = streamBrowser->getStreamParameter(StreamBrowser::streamAudioSps);
    m_audioSps = value + " KHz ";
    value = streamBrowser->getStreamParameter(StreamBrowser::streamAudioBps);
    m_audioBps = value + " Kbps ";

    
    bool videoActive = !( streamBrowser->getStreamParameter(StreamBrowser::streamVideoCodec) == "" );
    if ( videoActive )
    {
        value = "Video codec: " + streamBrowser->getStreamParameter(StreamBrowser::streamVideoCodec);
        streamDisplay->streamVideoFormat->setText( value );
        //value = streamBrowser->getStreamParameter(StreamBrowser::streamVideoFormat);
        m_videoQ = streamBrowser->getStreamParameter(StreamBrowser::streamVideoFps) + " Fps ";
        m_videoQ += streamBrowser->getStreamParameter(StreamBrowser::streamVideoBps) + " Bps ";
        m_videoQ += streamBrowser->getStreamParameter(StreamBrowser::streamVideoLength) + " s";
    }
       else streamDisplay->streamVideoFormat->setText("");

    value = "Audio: " + m_audioBps + m_audioSps + m_audioCh;
    if ( videoActive )
      value = value + "  Video: " + m_videoQ;
    
    //check if ( currentStreamStatus != "playing" )
    //  value = "";
    streamDisplay->streamAudioQuality->setText( value );
}

void StreamHandler::updateStreamPlayValues()
{
    QString value;
    bool converted;
    int gauge;

    value = streamBrowser->getStreamParameter(StreamBrowser::streamTime);
    streamDisplay->streamTime->setText( value );

    value = streamBrowser->getStreamParameter(StreamBrowser::streamCache);
    if ( value == "" ) value = "0";
    streamDisplay->streamCacheLabel->setText("buffer " + value + "%");
    gauge = value.toInt( &converted, 10 );
    if ( converted )
      streamDisplay->streamCacheGauge->setPercentage( gauge );

    value = streamBrowser->getStreamParameter(StreamBrowser::streamStability);
    if ( value == "" ) value = "0";
    streamDisplay->streamStabilityLabel->setText("stability " + value + "%");
    gauge = value.toInt( &converted, 10 );
    if ( converted )
      streamDisplay->streamStabilityGauge->setPercentage( gauge );

    for ( unsigned int i = 0;
          i < streamBrowser->getStreamCustomParameterCount();
          i++ )
    {
      value = streamBrowser->getStreamCustomParameter(i);
      streamDisplay->playerMessageBox->addMessage( value );
    }
}


//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
/*
ViewerHandler::ViewerHandler(BrowserDisplay *browserDisplay, StreamBrowser *streamBrowser)
           : GenericHandler()
{
    this->browserDisplay = browserDisplay;
    this->streamBrowser  = streamBrowser;  
    setTitleColor(yellow);
    dataLoaded = false;
}
      
bool ViewerHandler::handlePressedKey( int key )
{
   bool handled = true;
   
   if (key == inspect)
   {
     cout << "emit switchhandler" << endl;
     emit switchHandler(0); 
     return true;
   }   
    
   switch (key) 
   {
     case previtem:
        if (displaycursor > 0) 
          displaycursor--;
        else 
          if (listtop > 0)
            listtop--;  
        navigateItems();
     break;
     case nextitem:
        if ( (displaycursor < browserDisplay->getItemCount() -1) && 
              listtop + displaycursor < displist.size() -1 )
          displaycursor++;
        else 
          if ( listtop + browserDisplay->getItemCount() < displist.size() - 1 )
            listtop++;        
        navigateItems();
     break;
     case prevfolder:
     break;
     case nextfolder: 
     break;
     case selectitem:
     break;
     default:
       handled = false;
     break;
   }
   
   return handled;  // false: send key to streamBrowser
}

void ViewerHandler::activate()
{ 
   QString url, descr, handler;
   QStringList meta, metaparts;


   if (title == ""  ) title   = "-";
   if (url == ""    ) url     = "-";
   if (descr == ""  ) descr   = "-";
   if (handler == "") handler = "default";


   // create metalist, viewerlist
   metalist.clear();
   viewerlist.clear();
   displist.clear();
   
   displist.append("url: " + url);
   displist.append("descr: " + descr);
   displist.append("handler: " + handler);
   metalist.append("");   metalist.append("");   metalist.append("");
   viewerlist.append(""); viewerlist.append(""); viewerlist.append("");
      
   for (uint i = 0; i < meta.size(); i++)
   {
      metaparts = QStringList::split("(stsep)", *meta.at(i), true);
      if (metaparts.size() < 3)
      {
        displist.append("unknown");
        viewerlist.append("");
        metalist.append("");
      }
        else
      {
        displist.append(*metaparts.at(0));
        viewerlist.append(*metaparts.at(1));
        metalist.append(*metaparts.at(2));
      }
   }
   
   displaycursor  = 0;
   listtop        = 0;
            
   browserDisplay->show();
   navigateItems();
}

void ViewerHandler::deactivate()
{
   browserDisplay->hide();
}

void ViewerHandler::setTitleColor( QColor color )
{
   browserDisplay->setTitleColor( color, true );   
}

void ViewerHandler::navigateItems()
{
    QString caption, temp;
    QChar prefix;
    int cnt, size;

    if (!dataLoaded) return;
        
    size = displist.size();
    caption = "Showing details of: " + title;
    
    browserDisplay->setTitleLabel( caption );  
    cnt = browserDisplay->getItemCount();

    for (int i = 0; i < cnt; i++ )
    {
      prefix = '.';
      
      if (i + listtop < size)
      {
        caption = *displist.at(i + listtop);
        temp = *viewerlist.at(i);
        if (temp == "html")
          prefix = '_';
      }
      else
      {
        caption = "";
        prefix  = ' ';
      }

         
      browserDisplay->setItemIcon( i, prefix );
      browserDisplay->setItemLabel( i, caption, false );
    }
    
    if ( displaycursor >= 0 )
    {
      caption = QString::null;  
      browserDisplay->setItemLabel( displaycursor, caption, true );
    }

}
*/
